# Copyright 2026 Invoke Labs, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""LLM provider wrapping to implement Voker tracking.

Do not auto import this module directly. Instead, import from specific provider modules, e.g.,
`voker.ai.provider_openai`, `voker.ai.provider_gemini`, or `voker.ai.provider_anthropic`. This way
users won't need to install providers they don't use.
"""
