# Copyright 2026 Invoke Labs, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Person related payloads and responses."""

from typing import Any, NotRequired, Required, TypedDict


class CreatePersonPayload(TypedDict):
    """Create a person.

    See [Creating a Person](https://app.voker.ai/docs/concepts/people#creating-a-person) for more information.
    """

    person_id: Required[str]
    properties: NotRequired[dict[str, Any]]


class UpdatePersonPayload(TypedDict):
    """Update a person."""

    properties: Required[dict[str, Any]]
